@@ -11,7 +11,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: discovery.c,v 1.25 2006/01/03 03:20:38 dfs Exp $";
+"$Id$";
 
 #include "pppoe.h"
 
@@ -26,6 +26,7 @@ static char const RCSID[] =
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#include <time.h>
 
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
@@ -322,6 +323,9 @@ waitForPADO(PPPoEConnection *conn, int timeout)
     fd_set readable;
     int r;
     struct timeval tv;
+    struct timeval expire_at;
+    struct timeval now;
+
     PPPoEPacket packet;
     int len;
 
@@ -332,10 +336,31 @@ waitForPADO(PPPoEConnection *conn, int timeout)
     pc.seenACName    = 0;
     pc.seenServiceName = 0;
 
+    if (gettimeofday(&expire_at, NULL) < 0) {
+	fatalSys("gettimeofday (waitForPADO)");
+    }
+    expire_at.tv_sec += timeout;
+
     do {
 	if (BPF_BUFFER_IS_EMPTY) {
-	    tv.tv_sec = timeout;
-	    tv.tv_usec = 0;
+	    if (gettimeofday(&now, NULL) < 0) {
+		fatalSys("gettimeofday (waitForPADO)");
+	    }
+	    tv.tv_sec = expire_at.tv_sec - now.tv_sec;
+	    tv.tv_usec = expire_at.tv_usec - now.tv_usec;
+	    if (tv.tv_usec < 0) {
+		tv.tv_usec += 1000000;
+		if (tv.tv_sec) {
+		    tv.tv_sec--;
+		} else {
+		    /* Timed out */
+		    return;
+		}
+	    }
+	    if (tv.tv_sec <= 0 && tv.tv_usec <= 0) {
+		/* Timed out */
+		return;
+	    }
 
 	    FD_ZERO(&readable);
 	    FD_SET(conn->discoverySocket, &readable);
@@ -347,7 +372,10 @@ waitForPADO(PPPoEConnection *conn, int timeout)
 	    if (r < 0) {
 		fatalSys("select (waitForPADO)");
 	    }
-	    if (r == 0) return;        /* Timed out */
+	    if (r == 0) {
+		/* Timed out */
+		return;
+	    }
 	}
 
 	/* Get the packet */
@@ -508,13 +536,37 @@ waitForPADS(PPPoEConnection *conn, int timeout)
     fd_set readable;
     int r;
     struct timeval tv;
+    struct timeval expire_at;
+    struct timeval now;
+
     PPPoEPacket packet;
     int len;
 
+    if (gettimeofday(&expire_at, NULL) < 0) {
+	fatalSys("gettimeofday (waitForPADS)");
+    }
+    expire_at.tv_sec += timeout;
+
     do {
 	if (BPF_BUFFER_IS_EMPTY) {
-	    tv.tv_sec = timeout;
-	    tv.tv_usec = 0;
+	    if (gettimeofday(&now, NULL) < 0) {
+		fatalSys("gettimeofday (waitForPADS)");
+	    }
+	    tv.tv_sec = expire_at.tv_sec - now.tv_sec;
+	    tv.tv_usec = expire_at.tv_usec - now.tv_usec;
+	    if (tv.tv_usec < 0) {
+		tv.tv_usec += 1000000;
+		if (tv.tv_sec) {
+		    tv.tv_sec--;
+		} else {
+		    /* Timed out */
+		    return;
+		}
+	    }
+	    if (tv.tv_sec <= 0 && tv.tv_usec <= 0) {
+		/* Timed out */
+		return;
+	    }
 
 	    FD_ZERO(&readable);
 	    FD_SET(conn->discoverySocket, &readable);
@@ -526,7 +578,10 @@ waitForPADS(PPPoEConnection *conn, int timeout)
 	    if (r < 0) {
 		fatalSys("select (waitForPADS)");
 	    }
-	    if (r == 0) return;
+	    if (r == 0) {
+		/* Timed out */
+		return;
+	    }
 	}
 
 	/* Get the packet */
@@ -596,15 +651,6 @@ discovery(PPPoEConnection *conn)
     int padrAttempts = 0;
     int timeout = conn->discoveryTimeout;
 
-    /* Skip discovery and don't open discovery socket? */
-    if (conn->skipDiscovery && conn->noDiscoverySocket) {
-	conn->discoveryState = STATE_SESSION;
-	return;
-    }
-
-    conn->discoverySocket =
-	openInterface(conn->ifName, Eth_PPPOE_Discovery, conn->myEth);
-
     /* Skip discovery? */
     if (conn->skipDiscovery) {
 	conn->discoveryState = STATE_SESSION;
